@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2009  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2009, 2010  Internet Systems Consortium, Inc. ("ISC")
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,7 +14,7 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: clean.sh,v 1.2.14.3 2009/12/30 08:55:48 jinmei Exp $
+# $Id: clean.sh,v 1.2.4.5 2010-12-18 23:46:09 tbox Exp $
 
 rm -rf */*.signed
 rm -rf */*.jnl
@@ -28,3 +28,7 @@ rm -rf ns2/example.db
 rm -rf ns2/example.com.db
 rm -rf random.data
 rm -rf nsupdate.out.test
+rm -f ns1/keyset-.
+rm -f ns2/keyset-example.
+rm -f ns2/keyset-example.com.
+
